/*
 * Timing and control for writing Z-lookup tables
 * and for blanking display
 */

input	pixel_freq;
active_low input
	access_zlut, fifo_empty, blank;
active_high input
	fifo[17:16], zainc;

active_low output clocked_by (pixel_freq)
	fifo_rd, zlut_we, ld_zctr, red_cs, grn_cs, blu_cs,
	pixel_oe, data_oe, clr, ef_s, y;
active_high output clocked_by (pixel_freq)
	data_latch;
active_low output
	zctr_clk;

/* pixel_oe and data_oe are complementary; each waits for the other
   before being asserted.  data_oe stays asserted while there is a
   write access in progress. */
pixel_oe := ~access_zlut & ~data_oe;
data_oe := access_zlut & ~pixel_oe | fifo_rd | zlut_we;

/* fifo_rd starts a write access cycle.  It is followed by zlut_we
   and then by one of {red,grn,blu}_cs or ld_zctr. */
ef_s := fifo_empty;			/* synchronize it */
fifo_rd := access_zlut & ~ef_s & ~fifo_rd	/* cycle 1 */
	| fifo_rd & ~zlut_we;			/* cycle 2 */
zlut_we := fifo_rd;				/* cycles 2 & 3 */
data_latch := fifo_rd & ~zlut_we;		/* cycle 2 */

/* cs signals to banks are asserted during cycle 3 of a write, and
   during active display time. */
red_cs := fifo_rd & zlut_we & (fifo[17:16] == <01>)
	| ~access_zlut & ~data_oe;
grn_cs := fifo_rd & zlut_we & (fifo[17:16] == <10>)
	| ~access_zlut & ~data_oe;
blu_cs := fifo_rd & zlut_we & (fifo[17:16] == <11>)
	| ~access_zlut & ~data_oe;

/* The Zlut address reg is incremented or loaded on the trailing edge
   of zctr_clk, which follows blu_cs during a write to the blue Z-lut
   (also red and green if zainc is asserted), and is asserted during
   cycle 3 of a write to the address reg (in which case ld_zctr is
   asserted during cycles 3 & 4). */
zctr_clk = data_oe & (blu_cs | zainc & (red_cs | grn_cs))
	| ld_zctr & zlut_we;
ld_zctr := zlut_we & (fifo_rd & (fifo[17:16] == <00>))	/* cycle 3 */
		| ld_zctr;				/* cycle 4 */

/* clr follows blank by two cycles on the leading (asserting) edge,
   and by one cycle on the trailing edge.  This is necessary because
   it is an asynchronous clear. */
y := blank;
clr := y & blank;

define_pal zlut_1 = "gal16v8" {
	input access_zlut (2), fifo[17:16] (3:4), zainc (5);
	input data_oe (6), fifo_empty (7);
	output fifo_rd (19), zlut_we (18), ld_zctr (17);
	output red_cs (16), grn_cs (15), blu_cs (14);
	output ef_s (13), zctr_clk (12);
};

define_pal zlut_2 = "gal16v8" {
	input fifo_rd (2), zlut_we (3), access_zlut (4), blank (5);
	output data_latch (19), pixel_oe (18), data_oe (17);
	output clr (16), y (15);
};
